from __future__ import annotations
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef


class Container(Element):
    """
    Creates a layout container element with configurable width and spacing properties.

    A flexible container component for structuring content layouts, providing controls for
    maximum width and internal padding. The container is horizontally centered by default
    through auto margins.

    Args:
        max_width (TMaybeRef[str], optional): Sets the maximum width of the container.
            Accepts CSS width values (e.g., "800px", "100%", "75vw"). Defaults to "800px".
        padding (TMaybeRef[str], optional): Controls internal spacing between container
            edges and content. Accepts CSS padding values (e.g., "0.25rem", "1em", "10px").
            Defaults to "0.25rem".
    """

    def __init__(
        self,
        max_width: TMaybeRef[str] = "800px",
        *,
        padding: TMaybeRef[str] = "0.25rem",
    ):
        super().__init__("div")

        self.style({"max-width": max_width, "padding": padding}).style(
            "margin-left:auto;margin-right:auto"
        )



from __future__ import annotations
from pathlib import Path
from typing import Any, Dict, Union
from dataclasses import dataclass, field


@dataclass
class JsLink:
    link: Union[str, Path]
    attrs: Dict[str, Any] = field(default_factory=dict)

    def create_attrs_str(self):
        return " ".join(f'{k}="{v}"' for k, v in self.attrs.items() if v is not None)


@dataclass(frozen=True)
class VueAppUse:
    name: str


@dataclass(frozen=True)
class VueAppComponent:
    name: str
    url: str


@dataclass
class ImportMaps:
    name: str
    url: str



from __future__ import annotations
from typing import Any, Union
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui import ui_system_var_type as ui_vars_type


class Heading(Element):
    def __init__(
        self,
        text: Union[str, TMaybeRef[Any]],
        *,
        size: Union[TMaybeRef[str], ui_vars_type.TSize, None] = None,
        weight: Union[TMaybeRef[str], ui_vars_type.TWeight, None] = None,
        align: Union[TMaybeRef[str], ui_vars_type.TAlign, None] = None,
    ):
        """
        Creates a heading element with customizable text content and styling properties.


        Args:
            text (Union[str, TMaybeRef[Any]]): The text content of the heading. Can be a static
                string or a reactive reference (e.g., state object).
            size (Union[TMaybeRef[str], TSize, None], optional): Controls the heading size.
                Accepts values from "1" to "9". Defaults to None.
            weight (Union[TMaybeRef[str], TWeight, None], optional): Sets font weight.
                Acceptable values: "light", "regular", "medium", "bold". Defaults to None.
            align (Union[TMaybeRef[str], TAlign, None], optional): Controls text alignment.
                Valid options: "left", "center", "right". Defaults to None.

        Example:
        .. code-block:: python
            size = ui.state("6")
            text = ui.state("test")
            align = ui.state("left")

            html.select.from_list(["1", "2", "3", "4", "5", "6", "7", "8", "9"], size)
            html.select.from_list(["left", "center", "right"], align)

            with ui.container():
                html.input(text)
                ui.heading(text, size=size, align=align)
        """
        super().__init__("heading")

        self.props({"text": text, "size": size, "weight": weight, "align": align})



from __future__ import annotations
from typing import Dict
from instaui.common.jsonable import Jsonable
from instaui.runtime._app import get_current_scope
from instaui.vars._types import OutputSetType
from instaui.vars.mixin_types.py_binding import CanOutputMixin
from instaui.vars.mixin_types.element_binding import ElementBindingMixin


class ElementRef(Jsonable, CanOutputMixin, ElementBindingMixin):
    def __init__(self) -> None:
        scope = get_current_scope()
        self.__register_info = scope.register_element_ref_task(self)

    def __to_binding_config(
        self,
    ):
        return {
            "id": self.__register_info.var_id,
            "sid": self.__register_info.scope_id,
        }

    def _to_output_config(self):
        return self.__to_binding_config()

    def _to_json_dict(self):
        data = super()._to_json_dict()
        data["id"] = self.__register_info.var_id

        return data

    def _to_element_binding_config(self) -> Dict:
        return self.__to_binding_config()

    def _to_event_output_type(self) -> OutputSetType:
        return OutputSetType.ElementRefAction


def run_element_method(method_name: str, *args, **kwargs):
    return {
        "method": method_name,
        "args": args,
    }



from __future__ import annotations
from typing import Optional, Union
from instaui.components.element import Element


class Icon(Element):
    def __init__(
        self,
        icon: Optional[str] = None,
        *,
        size: Optional[Union[int, str]] = None,
        color: Optional[str] = None,
        svg_name: Optional[str] = None,
        raw_svg: Optional[str] = None,
    ):
        """
        Creates an icon component by referencing an SVG symbol from a local file.

        By default, the icon is loaded from a predefined directory structure:
        ```
        - assets/
          - icons/
            - icons.svg  # Contains SVG symbols with IDs matching `icon` names
        - main.py      # Entry point of the application
        ```
        If `svg_name` are not specified, the component will
        automatically resolve to `assets/icons/icons.svg`.

        Args:
            icon (Optional[str]): The name of the icon to display. This must match the `id`
                        of an SVG `<symbol>` in the target SVG file.
            size (Optional[Union[int, str]]): The size of the icon in pixels or CSS units.
                                              Defaults to None (natural size).
            color (Optional[str]): The color of the icon. Defaults to None (inherits text color).
            svg_name (Optional[str]): The filename of the SVG sprite. Defaults to None
                                     (resolves to `icons.svg`).
            raw_svg (Optional[str]): The raw SVG code to use instead of loading from a file.

        Example:
        .. code-block:: python
            # Renders the SVG symbol with ID "icon-1" from `assets/icons/icons.svg`
            ui.icon("icon-1")

            # Renders with custom size and color
            ui.icon("icon-2", size=24, color="#f00")
        """
        super().__init__("icon")

        self.props(
            {
                "icon": icon,
                "size": size,
                "color": color,
                "svgName": svg_name,
                "rawSvg": raw_svg,
            }
        )



from dataclasses import dataclass, field
from typing import Callable, List, Mapping, Optional

import pydantic_core
from instaui.systems import func_system, pydantic_system
from instaui.runtime.context import get_context


def create_handler_key(
    page_path: str,
    handler: Callable,
):
    _, lineno, _ = func_system.get_function_location_info(handler)

    if get_context().debug_mode:
        return f"page:{page_path}|line:{lineno}"

    return f"{page_path}|{lineno}"


@dataclass
class HandlerInfo:
    fn: Callable
    fn_location_info: str
    outputs_binding_count: int = 0
    handler_param_converters: List[pydantic_system.TypeAdapterProtocol] = field(
        default_factory=list
    )
    is_last_param_args: bool = False

    def get_handler_args(self, input_values: List):
        real_param_converters = _try_expand_params_converters(
            self.handler_param_converters, input_values, self.is_last_param_args
        )

        try:
            return [
                param_converter.to_python_value(value)
                for param_converter, value in zip(real_param_converters, input_values)
            ]
        except pydantic_core._pydantic_core.ValidationError as e:
            raise ValueError(f"invalid input[{self.fn_location_info}]: {e}") from None

    @classmethod
    def from_handler(
        cls,
        handler: Callable,
        outputs_binding_count: int,
        custom_type_adapter_map: Optional[
            Mapping[int, pydantic_system.TypeAdapterProtocol]
        ] = None,
    ):
        custom_type_adapter_map = custom_type_adapter_map or {}
        params_infos = func_system.get_fn_params_infos(handler)
        is_last_param_args = func_system.is_last_param_args(handler)
        param_converters = [
            custom_type_adapter_map.get(
                idx, pydantic_system.create_type_adapter(param_type)
            )
            for idx, (_, param_type) in enumerate(params_infos)
        ]

        file, lineno, _ = func_system.get_function_location_info(handler)

        return cls(
            handler,
            f'File "{file}", line {lineno}',
            outputs_binding_count,
            handler_param_converters=param_converters,
            is_last_param_args=is_last_param_args,
        )


def _try_expand_params_converters(
    old_param_converters: List[pydantic_system.TypeAdapterProtocol],
    input_values: List,
    is_last_param_args: bool,
):
    if not is_last_param_args:
        return old_param_converters

    diff = len(input_values) - len(old_param_converters)
    if diff == 0:
        return old_param_converters

    arg_param_converters = [old_param_converters[-1]] * diff

    return [*old_param_converters[:], *arg_param_converters]



from __future__ import annotations
import typing
from pydantic import BaseModel, Field, ConfigDict, field_serializer, model_serializer

from instaui.components.html.div import Div
from instaui.runtime._app import get_app_slot, get_current_scope
from instaui.components.component import Component
from instaui.common.jsonable import dumps2dict

from . import _types


class RouteItem(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    component_fn: typing.Optional[typing.Callable] = Field(exclude=True)
    sid: typing.Optional[str] = None
    vue_route_item: VueRouteItem = Field(serialization_alias="vueItem")
    meta: typing.Optional[typing.Dict] = None

    @model_serializer
    def model_ser(self):
        if self.component_fn is None and (not self.vue_route_item.path):
            raise ValueError("Either component_fn or vue_route_item.path must be set")

        if self.component_fn is None:
            return {
                "vueItem": self.vue_route_item,
            }

        if self.vue_route_item.path is None:
            self.vue_route_item.path = f"/{'' if self.component_fn.__name__ == 'index' else self.component_fn.__name__}"

        app = get_app_slot()
        with Div() as div:
            self.component_fn()
            scope_id = get_current_scope().id

        app.items.pop()

        self.vue_route_item.component = div._slot_manager.default._children

        return {
            "sid": scope_id,
            "vueItem": self.vue_route_item,
        }

    @classmethod
    def create(
        cls,
        *,
        component_fn: typing.Optional[typing.Callable] = None,
        path: typing.Optional[str] = None,
        name: typing.Optional[str] = None,
        params: typing.Optional[typing.Dict[str, str]] = None,
        children: typing.Optional[typing.List[RouteItem]] = None,
        meta: typing.Optional[typing.Dict] = None,
    ):
        """Create a new RouteItem

        Examples:
        .. code-block:: python
            routes = [
                spa_router.RouteItem.create(path='/',component_fn=home),
                spa_router.RouteItem.create(path='/user',component_fn=user_home),
            ]

            spa_router.config_router(routes=routes)

        Args:
            component_fn (typing.Callable): function that returns a component to be rendered.
            path (typing.Optional[str], optional): route path. Defaults to None.
            name (typing.Optional[str], optional): route name. Defaults to None.
            params (typing.Optional[typing.Dict[str, str]], optional): route params. Defaults to None.
            children (typing.Optional[typing.List[RouteItem]], optional): child routes. Defaults to None.

        """

        return cls(
            component_fn=component_fn,
            meta=meta,
            vue_route_item=VueRouteItem(
                path=path,
                name=name,
                params=params,
                component=[],
                children=children,
            ),
        )


class VueRouteItem(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    path: typing.Optional[str] = None
    params: typing.Optional[typing.Dict[str, str]] = None
    name: typing.Optional[str] = None
    component: typing.Optional[typing.List[Component]] = []
    children: typing.Optional[typing.List[RouteItem]] = None

    @field_serializer("component")
    def serialize_component(self, value: typing.List[Component]):
        if not value:
            return []
        return dumps2dict(value)


class RouteCollector(BaseModel):
    mode: _types.TRouterHistoryMode = "hash"
    keep_alive: bool = Field(default=False, serialization_alias="kAlive")
    routes: typing.List[RouteItem] = []

    def add_route(self, item: RouteItem):
        self.routes.append(item)
        return self



from instaui.runtime._app import get_app_slot
from instaui.spa_router._route_model import RouteCollector


def try_register_router_collector():
    app = get_app_slot()
    if app._route_collector is not None:
        return

    rb = RouteCollector()
    app.register_router(rb)



import inspect
import typing
from typing_extensions import ParamSpec
from instaui.common.jsonable import Jsonable
from instaui.runtime._app import get_current_scope, get_app_slot
from instaui.vars.mixin_types.py_binding import (
    CanInputMixin,
    CanOutputMixin,
    _assert_outputs_be_can_output_mixin,
    inputs_to_config,
    outputs_to_config,
)
from instaui.handlers import event_handler
from instaui import pre_setup as _pre_setup
from .event_mixin import EventMixin


_SYNC_TYPE = "sync"
_ASYNC_TYPE = "async"

P = ParamSpec("P")
R = typing.TypeVar("R")


class WebEvent(Jsonable, EventMixin, typing.Generic[P, R]):
    def __init__(
        self,
        fn: typing.Callable[P, R],
        inputs: typing.Sequence[CanInputMixin],
        outputs: typing.Sequence[CanOutputMixin],
        pre_setup: typing.Optional[typing.Dict] = None,
    ):
        if pre_setup:
            _pre_setup._check_args(pre_setup)

        _assert_outputs_be_can_output_mixin(outputs)

        self._inputs = inputs
        self._outputs = outputs
        self._fn = fn
        self._pre_setup = pre_setup

        scope = get_current_scope()
        self._sid = scope.id

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
        return self._fn(*args, **kwargs)

    def copy_with_extends(self, extends: typing.Sequence[CanInputMixin]):
        return WebEvent(
            fn=self._fn,
            inputs=list(self._inputs) + list(extends),
            outputs=self._outputs,
        )

    def event_type(self):
        return "web"

    def _to_json_dict(self):
        app = get_app_slot()

        hkey = event_handler.create_handler_key(
            page_path=app.page_path, handler=self._fn
        )

        event_handler.register_event_handler(
            hkey, self._fn, self._outputs, self._inputs
        )

        data = {}
        data["type"] = self.event_type()
        data["fType"] = (
            _ASYNC_TYPE if inspect.iscoroutinefunction(self._fn) else _SYNC_TYPE
        )
        data["hKey"] = hkey
        data["sid"] = self._sid

        if self._inputs:
            data["inputs"] = inputs_to_config(self._inputs)

        if self._outputs:
            data["sets"] = outputs_to_config(self._outputs)

        if self._pre_setup:
            data["preSetup"] = _pre_setup.convert_config(self._pre_setup)

        return data


def event(
    *,
    inputs: typing.Optional[typing.Sequence] = None,
    outputs: typing.Optional[typing.Sequence] = None,
    pre_setup: typing.Optional[typing.Dict] = None,
):
    """
    Creates an event handler decorator for binding reactive logic to component events.

    Args:
        inputs (typing.Optional[typing.Sequence], optional): Reactive sources (state objects, computed properties)
                                   that should be accessible during event handling.
                                   These values will be passed to the decorated function
                                   when the event fires.
        outputs (typing.Optional[typing.Sequence], optional): Targets (state variables, UI elements) that should
                                    update when this handler executes. Used for coordinating
                                    interface updates after the event is processed.
        pre_setup (typing.Optional[typing.Dict], optional): A dictionary of pre-setup actions to be executed before the event executes.


    # Example:
    .. code-block:: python
        from instaui import ui, html

        a = ui.state(0)

        @ui.event(inputs=[a], outputs=[a])
        def plus_one(a):
            return a + 1

        html.button("click me").on_click(plus_one)
        html.paragraph(a)

    use pre_setup:
    .. code-block:: python
        a = ui.state(0)
        task_running = ui.state(False)

        @ui.event(inputs=[a], outputs=[a], pre_setup={task_running: True})
        async def long_running_task(a):
            await asyncio.sleep(3)
            return a + 1

        html.button("click me").on_click(long_running_task).disabled(task_running)
    """

    def wrapper(func: typing.Callable[P, R]):
        return WebEvent(
            func,
            inputs or [],
            outputs=outputs or [],
            pre_setup=pre_setup,
        )

    return wrapper



from typing import Dict, Sequence, cast
from instaui.common.jsonable import Jsonable
from instaui.vars.mixin_types.py_binding import CanOutputMixin, CanInputMixin


def _check_args(config: Dict):
    for key in config.keys():
        if not isinstance(key, CanOutputMixin):
            raise TypeError(f"key {key} is not a CanOutputMixin")


def convert_config(config: Dict):
    return [
        {
            "target": cast(CanInputMixin, key)._to_input_config(),
            **value._to_json_dict(),
        }
        if isinstance(value, PreSetupAction)
        else {
            "type": "const",
            "target": cast(CanInputMixin, key)._to_input_config(),
            "value": value,
        }
        for key, value in config.items()
    ]


class PreSetupAction(Jsonable):
    def __init__(self, *, inputs: Sequence, code: str, reset: bool = True):
        self.type = "action"
        self._inputs = inputs
        self.code = code
        self.reset = reset

    def _to_json_dict(self):
        data = super()._to_json_dict()
        if self._inputs:
            data["inputs"] = [
                binding._to_input_config()
                if isinstance(binding, CanInputMixin)
                else binding
                for binding in self._inputs
            ]

        return data



from __future__ import annotations
import threading
from typing import (
    Callable,
    Dict,
    Generic,
    TypeVar,
)
from dataclasses import dataclass

from instaui.runtime.context import get_context
from instaui.runtime._app import update_web_server_info
from instaui.systems import func_system
from . import _utils


ASYNC_URL = "/instaui/watch/async"
SYNC_URL = "/instaui/watch/sync"

update_web_server_info(watch_url=SYNC_URL, watch_async_url=ASYNC_URL)

_watch_handlers: Dict[str, _utils.HandlerInfo] = {}
dict_lock = threading.Lock()


def register_handler(key: str, handler: Callable, outputs_binding_count: int):
    if key in _watch_handlers:
        return
    with dict_lock:
        _watch_handlers[key] = _utils.HandlerInfo.from_handler(
            handler, outputs_binding_count
        )


def get_handler_info(key: str) -> _utils.HandlerInfo:
    return _watch_handlers.get(key)  # type: ignore


def get_statistics_info():
    return {
        "_watch_handlers count": len(_watch_handlers),
        "_watch_handlers keys": list(_watch_handlers.keys()),
    }


def create_handler_key(page_path: str, handler: Callable):
    _, lineno, _ = func_system.get_function_location_info(handler)

    if get_context().debug_mode:
        return f"path:{page_path}|line:{lineno}"
    return f"{page_path}|{lineno}"


_TWatchStateValue = TypeVar("_TWatchStateValue")


@dataclass(frozen=True)
class WatchState(Generic[_TWatchStateValue]):
    new_value: _TWatchStateValue
    old_value: _TWatchStateValue
    modified: bool



from __future__ import annotations
from typing import (
    Dict,
)
from instaui.common.jsonable import Jsonable
from instaui.vars._types import InputBindingType
from instaui.vars.path_var import PathVar
from .mixin_types.py_binding import CanInputMixin
from .mixin_types.element_binding import ElementBindingMixin
from .mixin_types.pathable import CanPathPropMixin


class BindingSlotPropItem(
    Jsonable,
    PathVar,
    ElementBindingMixin,
    CanInputMixin,
    CanPathPropMixin,
):
    def __init__(self, name: str, sid: str, var_id: str) -> None:
        super().__init__()
        self._name = name
        self._sid = sid
        self._id = var_id

    def __to_binding_config(self):
        data: Dict = {
            "sid": self._sid,
            "id": self._id,
        }

        return data

    def _to_element_binding_config(self):
        return self.__to_binding_config()

    def _to_input_config(self):
        return self.__to_binding_config()

    def _to_path_prop_binding_config(self) -> Dict:
        return self.__to_binding_config()

    def _to_pathable_binding_config(self) -> Dict:
        return self.__to_binding_config()

    def _to_json_dict(self):
        data: Dict = {
            "id": self._id,
        }

        return data

    def _to_event_input_type(self) -> InputBindingType:
        return InputBindingType.Ref



from __future__ import annotations
from typing import (
    Callable,
    Dict,
    Generic,
    List,
    Literal,
    Optional,
    Tuple,
    TypeVar,
    Hashable,
)

_TLinkActionObj = TypeVar("_TLinkActionObj")

_DEFAULT_KEY_FN = lambda x: x  # noqa: E731


class LinkManager(Generic[_TLinkActionObj]):
    def __init__(
        self, key_fn: Optional[Callable[[_TLinkActionObj], Hashable]] = None
    ) -> None:
        self._key_fn = key_fn or _DEFAULT_KEY_FN
        self._actions_map: Dict[
            Tuple[Literal["add", "remove"], Hashable], _TLinkActionObj
        ] = {}

    def add_link(
        self,
        obj: _TLinkActionObj,
    ):
        key = ("add", self._key_fn(obj))
        self._actions_map[key] = obj

    def remove_link(
        self,
        obj: _TLinkActionObj,
    ):
        key = ("remove", self._key_fn(obj))
        self._actions_map[key] = obj

    def get_valid_links(
        self, secondary_manager: Optional[LinkManager] = None
    ) -> List[_TLinkActionObj]:
        secondary_maps = secondary_manager._actions_map if secondary_manager else {}
        merger_map = {**secondary_maps, **self._actions_map}

        result = {}

        for (action, key), obj in merger_map.items():
            if action == "add":
                result[key] = obj
                continue

            if key in result:
                del result[key]

        return list(result.values())


if __name__ == "__main__":

    def test_baes():
        manager: LinkManager[str] = LinkManager()
        manager.add_link("a")
        manager.add_link("b")
        manager.add_link("c")
        manager.remove_link("b")

        assert manager.get_valid_links() == ["a", "c"]

    test_baes()

    def test_with_other():
        other: LinkManager[str] = LinkManager()
        other.add_link("a")
        other.add_link("b")
        other.add_link("c")

        manager1: LinkManager[str] = LinkManager()
        manager1.add_link("b")
        manager1.add_link("d")
        manager1.add_link("e")
        manager1.remove_link("a")
        manager1.remove_link("b")

        assert manager1.get_valid_links(other) == ["c", "d", "e"]

    test_with_other()



from __future__ import annotations
from typing import Literal
from instaui.components.element import Element
from instaui.vars.types import TMaybeRef
from instaui.vars.js_computed import JsComputed


class Column(Element):
    def __init__(self, *, inline: TMaybeRef[bool] = False):
        super().__init__("div")

        flex = JsComputed(
            inputs=[inline], code="inline => inline? 'inline-flex' : 'flex'"
        )

        self.style("flex-direction: column;gap:var(--insta-column-gap)").style(
            {"display": flex}
        )

    def gap(self, value: TMaybeRef[str]) -> Column:
        return self.style({"gap": value})

    def align_items(
        self, value: TMaybeRef[Literal["start", "end", "center", "stretch", "revert"]]
    ) -> Column:
        return self.style({"align-items": value})



from __future__ import annotations
import typing
from dataclasses import dataclass, field
from instaui.common.jsonable import dumps
from .env import env

if typing.TYPE_CHECKING:
    from instaui.runtime.resource import StyleTag


_html_template = env.get_template("webview.html")


@dataclass(frozen=True)
class WebViewVueAppComponent:
    name: str
    url: str


@dataclass
class WebViewTemplateModel:
    version: str
    vue_js_code: str
    instaui_js_code: str
    config_dict: typing.Dict[str, typing.Any] = field(default_factory=dict)
    extra_import_maps: typing.Dict[str, str] = field(default_factory=dict)
    css_links: typing.List[str] = field(default_factory=list)
    style_tags: typing.List[StyleTag] = field(default_factory=list)
    js_links: typing.List[str] = field(default_factory=list)
    script_tags: typing.List[str] = field(default_factory=list)
    vue_app_use: typing.List[str] = field(default_factory=list)
    vue_app_component: typing.List[WebViewVueAppComponent] = field(default_factory=list)
    title: typing.Optional[str] = None
    favicon_url: typing.Optional[str] = None
    on_app_mounted: typing.Optional[typing.Callable] = None

    def add_extra_import_map(self, name: str, code: str):
        self.extra_import_maps[name] = code

    @property
    def import_maps_string(self):
        data = {
            **self.extra_import_maps,
            "vue": self.vue_js_code,
            "instaui": self.instaui_js_code,
        }

        return dumps(data)


def render_wbeview_html(model: WebViewTemplateModel) -> str:
    return _html_template.render(model=model)



from __future__ import annotations

import typing
from instaui.components.component import Component

from instaui.vars.mixin_types.element_binding import ElementBindingProtocol


class Match(Component):
    def __init__(self, on: ElementBindingProtocol):
        super().__init__("match")
        self._on = on
        self.cass_id_count = -1
        self._case_values = []

    def case(self, value: typing.Any):
        self.cass_id_count += 1
        self._case_values.append(value)
        return self._slot_manager.get_slot(str(self.cass_id_count))

    def default(self):
        return self._slot_manager.get_slot(":default")

    def _to_json_dict(self):
        data = super()._to_json_dict()
        data["on"] = self._on._to_element_binding_config()
        data["caseValues"] = self._case_values

        data["slots"] = {
            name: slot
            for name, slot in self._slot_manager._slots.items()
            if name != ":"
        }

        default_slot = self._slot_manager.get_slot(
            "default"
        )._to_items_container_config()

        if "sid" in default_slot:
            data["sid"] = default_slot["sid"]

        return data
